/*
 * Tcp4u v 3.20          Last Revision 06/12/97  3.20
 *
 *===========================================================================
 *
 * Project: Tcp4u,      Library for tcp protocol
 * File:    tn4u.c
 * Purpose: telnet based-functions
 *
 *===========================================================================
 *
 * This software is Copyright (c) 1996-1997 by Philippe Jounin
 *
 * Permission is hereby granted to copy, distribute or otherwise
 * use any part of this package as long as you do not try to make
 * money from it or pretend that you wrote it.  This copyright
 * notice must be maintained in any copy made.
 *
 * Use of this software constitutes acceptance for use in an AS IS
 * condition. There are NO warranties with regard to this software.
 * In no event shall the author be liable for any damages whatsoever
 * arising out of or in connection with the use or performance of this
 * software.  Any use of this software is at the user's own risk.
 *
 *  If you make modifications to this software that you feel
 *  increases it usefulness for the rest of the community, please
 *  email the changes, enhancements, bug fixes as well as any and
 *  all ideas to me. This software is going to be maintained and
 *  enhanced as deemed necessary by the community.
 *
 *   Philippe Jounin (ph.jounin@computer.org)
 */


#include "build.h"


/* ******************************************************************* */
/*                                                                     */
/* Partie II : Lecture des trames de type Telnet                       */
/*                                                                     */
/* ******************************************************************* */

/* ------------------------------------------------------------ */
/* TnReadLine : Lit une ligne de donnes (jusqu' <LF>)         */
/*              retourne TN_ERROR, TN_TIMEOUT, TN_BUFFERFREED   */
/*              TN_OVERFLOW ou le nombre d'octets lus           */
/*              dans les deux derniers cas le buffer szBuf est  */
/*              renseign. Si TN_SUCCESS il est <LF>-termin.   */
/* ------------------------------------------------------------ */

int API4U TnReadLine (SOCKET s,LPSTR szBuf,UINT uBufSize,UINT uTimeOut,HFILE hf)
{
int Rc;

   Tcp4uLog (LOG4U_PROC, "TnReadLine");
   Rc = InternalTnReadLine (s, szBuf, uBufSize, uTimeOut, hf);
   if (Rc==TN_SUCCESS)   Tcp4uDump (szBuf, Strlen(szBuf), DUMP4U_RCVD);
   Tcp4uLog (LOG4U_EXIT, "TnReadLine");
return Rc;
} /* TnReadLine */


int InternalTnReadLine (SOCKET s,LPSTR szBuf,UINT uBufSize,UINT uTimeOut,
                        HFILE hf)
{
int Rc;

   if (s==INVALID_SOCKET || uBufSize==0)  return TN_ERROR;  /* ??? */

   memset (szBuf, 0, uBufSize);
   Rc = InternalTcpRecvUntilStr (s, szBuf, & uBufSize, "\n", 1, 
                                 TRUE, uTimeOut, hf);
   if (Rc>=TCP4U_SUCCESS  &&  szBuf[uBufSize-1]=='\r')  
         szBuf[uBufSize-1]=0 ;   
   /* pas de translation des erreurs, timeout... */
return  (Rc>=TCP4U_SUCCESS) ?  TN_SUCCESS  : Rc;
} /* InternalTnReadLine */





/* ******************************************************************* */
/*                                                                     */
/* Partie III : Etage Telnet Haut                                      */
/*                                                                     */
/* ******************************************************************* */

/* ------------------------------------------------------------ */
/* TnSend : Formatage d'une trame au format Telnet (termine    */
/*          par <RC><LF>                                        */
/*          Retour par TN_ERROR, ou TN_SUCCESS.                 */
/* ------------------------------------------------------------ */
int API4U TnSend (SOCKET s, LPCSTR szString, BOOL bHighPriority, HFILE hf)
{
int Rc, Z;
LPSTR p;

   Tcp4uLog (LOG4U_PROC, "TnSend");

   Z = Strlen (szString)+sizeof ("\r\n")-1;
   p = Calloc (Z+1, 1);
   if (p==NULL)  return TCP4U_INSMEMORY;
   Strcpy (p, szString);
   Strcat (p, "\r\n");
   Rc = InternalTcpSend (s, p, Z, bHighPriority, hf);
   Free (p);

   if (Rc==TCP4U_SUCCESS)   Tcp4uDump (p, Z, DUMP4U_SENT);
   Tcp4uLog (LOG4U_EXIT, "TnSend : return code %d", Rc);
return Rc==TCP4U_SUCCESS ?  TN_SUCCESS : TN_ERROR;
} /* TnSend */


/* ------------------------------------------------------------ */
/* TnReadMultiLine :  Attend une rponse du serveur e           */
/*                    La reponse peut contenir plusieurs lignes */
/*                    (depend du 4eme caractere)                */
/*                  - Retourne une erreur Telnet TN_ classiquea */
/* ------------------------------------------------------------ */
int API4U TnReadMultiLine (SOCKET ctrl_skt, LPSTR szInBuf,
                           UINT uBufSize, UINT uTimeOut, HFILE hf)
{
int      Rc, TnRc;
unsigned nPos=0;
   
   Tcp4uLog (LOG4U_PROC, "TnReadMultiLine");

   if (szInBuf==NULL || uBufSize==0)  return TN_ERROR;
   memset (szInBuf, 0, uBufSize);

   do
   {
       /* ajouter les fins de lignes */
       if (nPos!=0)
       {    
       		nPos+=Strlen (&szInBuf[nPos]);	/* pointer sur la fin de ligne */
       		szInBuf[nPos++]='\n';
       }
       Rc = InternalTnReadLine (ctrl_skt, &szInBuf[nPos], 
                                uBufSize-nPos-1, uTimeOut,hf);
    }
    while (    nPos < uBufSize - 5  /* encore de la place (y compris pour \0) */
            && Rc>=TCP4U_SUCCESS    /* retour correct */
            && szInBuf[nPos+4] == '-' );

    /* supprime la derniere marque fin de ligne */
    if (Rc>=TCP4U_SUCCESS)
    {                       
       Rc = TCP4U_SUCCESS;
       nPos+=Strlen (&szInBuf[nPos]);
       if (szInBuf[nPos-1]=='\r')   szInBuf[nPos-1]=0;
    }
    /* translation du code de retour */
    switch (Rc)
    {
       case TCP4U_TIMEOUT :      TnRc =  TN_TIMEOUT;		break;
       case TCP4U_SUCCESS :      Tcp4uDump (szInBuf, nPos, DUMP4U_RCVD);
                                 TnRc =  TN_SUCCESS;     	break;
       case TCP4U_OVERFLOW :     Tcp4uDump (szInBuf, nPos, "Overflow");
                                 TnRc =  TN_OVERFLOW; 		break;
       case TCP4U_SOCKETCLOSED : TnRc =  TN_SOCKETCLOSED;	break;
       case TCP4U_CANCELLED    : TnRc =  TN_CANCELLED;		break;
       default :                 TnRc =  TN_ERROR;			break;
    }

   Tcp4uLog (LOG4U_EXIT, "TnReadMultiLine: return code %d", TnRc);
return TnRc;
} /* TnReadMultiLine */


/* ------------------------------------------------------------ */
/* TnGetAnswerCode :  Attend une rponse du serveur et rend le  */
/*                    code numrique (les 3 premiers octets)    */
/*                    renvoy par le serveur.                   */
/*                  - Retourne un nombre entre 100 et 999 si la */
/*                    fonction s'est bien passe, sinon un code */
/*                    d'erreur TN_ERROR, TN_TIMEOUT...          */
/* ------------------------------------------------------------ */
int API4U TnGetAnswerCode (SOCKET ctrl_skt, LPSTR szInBuf,
                           UINT uBufSize, UINT uTimeOut, HFILE hf)
{
int      Rc;

   Tcp4uLog (LOG4U_PROC, "TnReadAnswerCode");
   
   Rc = TnReadMultiLine (ctrl_skt, szInBuf, uBufSize, uTimeOut, hf);
   if (Rc==TN_SUCCESS)		Rc = Tcp4uAtoi (szInBuf);
   
   Tcp4uLog (LOG4U_EXIT, "TnReadAnswerCode: return code %d", Rc);   
return Rc;
} /* TnGetAnswerCode */

