/*
 * Tcp4u v 3.2          Last Revision 06/12/97  3.20
 *
 *===========================================================================
 *
 * Project: Tcp4u,      Library for tcp protocol
 * File:    tn_proto.c
 * Purpose: telnet based-protocol transactions
 *
 *===========================================================================
 *
 * This software is Copyright (c) 1996-1997 by Philippe Jounin
 *
 * Permission is hereby granted to copy, distribute or otherwise
 * use any part of this package as long as you do not try to make
 * money from it or pretend that you wrote it.  This copyright
 * notice must be maintained in any copy made.
 *
 * Use of this software constitutes acceptance for use in an AS IS
 * condition. There are NO warranties with regard to this software.
 * In no event shall the author be liable for any damages whatsoever
 * arising out of or in connection with the use or performance of this
 * software.  Any use of this software is at the user's own risk.
 *
 *  If you make modifications to this software that you feel
 *  increases it usefulness for the rest of the community, please
 *  email the changes, enhancements, bug fixes as well as any and
 *  all ideas to me. This software is going to be maintained and
 *  enhanced as deemed necessary by the community.
 *
 *   Philippe Jounin (ph.jounin@computer.org)
 */


#include "build.h"

/* ------------------------------------------------------------------------- */
/* This function does the dirty job of any protocol a la telnet              */
/* it sends an order accepting variable length parameter, then waits for the */
/* response. The response is compare with an array if strings passed by the  */
/* calling function. The result is the matching integer if any or            */
/* TN_UNEXPECTED                                                             */
/* ------------------------------------------------------------------------- */
int TnProtoExchange (SOCKET s, 
		             struct S_TnProto  *tTranslation, 
                     int    nTabSize,
			         BOOL   bMultiLine,
			         BOOL   bCaseCmp,
                     LPSTR  szResponse, UINT uBufSize, 
			         UINT   uTimeout,
			         HFILE  hLogFile,
			         LPSTR  szCommande, ...)
{
char    szInternalBuffer [4096];
va_list args;
int     Rc;
int     Ark;

   if (szCommande != NULL)
   {
     va_start (args, szCommande);
     wsprintf (szInternalBuffer, szCommande, args);
     va_end (args);

     Rc = TnSend (s, szInternalBuffer, FALSE, hLogFile);
     if (Rc != TN_SUCCESS)   return Rc;
   }

   /* on attaque a differents niveaux selon le protocole */
   if (bMultiLine)
         Rc = TnReadMultiLine (s, szInternalBuffer, sizeof szInternalBuffer,
                               uTimeout, hLogFile); 
   else  Rc = TnReadLine (s, szInternalBuffer, sizeof szInternalBuffer,
                          uTimeout, hLogFile);
   if (Rc != TN_SUCCESS)  return Rc;
   
   /* recopier pour l'appelant */
   if (szResponse != NULL) 
      Strcpyn (szResponse, szInternalBuffer, uBufSize);
   for ( Ark=0 ;  Ark<nTabSize ;  Ark++)
   {
      if(bCaseCmp)
      {
        if (Tcp4uStrncasecmp (szInternalBuffer, tTranslation[Ark].szString, Strlen (tTranslation[Ark].szString)) == 0)
            return tTranslation[Ark].iCode;
      }
      else
      {
        if (strncmp (szInternalBuffer, tTranslation[Ark].szString, Strlen (tTranslation[Ark].szString)) == 0)
            return tTranslation[Ark].iCode;
      }
   }
return TN_UNEXPECTED;
} /* TnProtoExchange */

