/*
 * Tcp4u v 3.20         Last Revision 03/07/97  3.10-02
 *
 *===========================================================================
 *
 * Project: Tcp4u,      Library for tcp protocol
 * File:    util.c
 * Purpose: A few utilities functions
 *
 *===========================================================================
 *
 * This software is Copyright (c) 1996-1997 by Philippe Jounin
 *
 * Permission is hereby granted to copy, distribute or otherwise
 * use any part of this package as long as you do not try to make
 * money from it or pretend that you wrote it.  This copyright
 * notice must be maintained in any copy made.
 *
 * Use of this software constitutes acceptance for use in an AS IS
 * condition. There are NO warranties with regard to this software.
 * In no event shall the author be liable for any damages whatsoever
 * arising out of or in connection with the use or performance of this
 * software.  Any use of this software is at the user's own risk.
 *
 *  If you make modifications to this software that you feel
 *  increases it usefulness for the rest of the community, please
 *  email the changes, enhancements, bug fixes as well as any and
 *  all ideas to me. This software is going to be maintained and
 *  enhanced as deemed necessary by the community.
 *
 *   Philippe Jounin (ph.jounin@computer.org)
 */


#include "build.h"

/* this definition will be enough for our use */
#undef  toupper
#define toupper(x)  ( (x) & 0xDF )



/* ------------------------------------------------------------------------- */
/* Conversion d'un nom de machine en adresse IP                              */
/* Le nom est soit une "dotted address" soit un alias                        */ 
/* return : INADDR_NONE or IP address										 */ 
/* ------------------------------------------------------------------------- */
struct in_addr Tcp4uGetIPAddr (LPCSTR szHost)
{
struct hostent far *  lpHostEnt;
struct in_addr        sin_addr;

  Tcp4uLog (LOG4U_INTERN, "Tcp4uGetIPAddr host %s", szHost);

  sin_addr.s_addr = inet_addr (szHost); /* doted address */
  if (sin_addr.s_addr==INADDR_NONE)     /* si pas une doted address  */
  {                      /* regarder le fichier hosts */
      Tcp4uLog (LOG4U_CALL, "gethostbyname host %s", szHost);
      lpHostEnt = gethostbyname (szHost);
      if (lpHostEnt!=NULL)
         memcpy (& sin_addr.s_addr, lpHostEnt->h_addr, lpHostEnt->h_length);
      else       
      {
        Tcp4uLog (LOG4U_ERROR, "gethostbyname host %s", szHost);
      }

    }
return sin_addr;
} /* Tcp4uGetIPAddr */



/* -------------------------------------- */
/* a self-made atoi                       */
/* -------------------------------------- */

/* Note: ne pas utiliser la fonction isdigit: qui donne des */
/* resultats errones en Windows                             */
#define IsDigit(x)   ( (x)>='0' &&  (x)<='9')


int Tcp4uAtoi (LPCSTR p)
{
int n;
  Tcp4uLog (LOG4U_INTERN, "Tcp4uAtoi");

   for (n=0 ;  IsDigit (*p) ; p++)
     {
       n *= 10;
       n += (*p - '0');
     }
return n;   
} /* tcp4u_atoi */


long Tcp4uAtol (LPCSTR p)
{
long n;
  Tcp4uLog (LOG4U_INTERN, "Tcp4uAtol");

   for (n=0 ;  IsDigit (*p) ; p++)
     {
       n *= 10;
       n += (*p - '0');
     }
return n;   
} /* Tcp4uAtol */


/* a unix functon which is not available under Windows */
int Tcp4uStrncasecmp (LPCSTR s1, LPCSTR s2, int n)
{
     while (--n >= 0 &&  toupper (*s1)==toupper(*s2++))
     {
         if (*s1++ == '\0')  
                return 0;
     }
return  n < 0 ? 0 : toupper (*s1) - toupper(*--s2);
} /* Strncasecmp */

/* ------------------------------------------------------------------------- */
/* a case-insentive strstr                                                   */
/* ------------------------------------------------------------------------- */
LPSTR Tcp4uStrIStr (LPCSTR s1, LPCSTR s2)
{
int nLength = Strlen(s2);
    for (;;)
    {
       while (*s1!=0  &&  toupper(*s1)!=toupper(*s2) )  *s1++;
       if (*s1==0)   return NULL;
       if (Tcp4uStrncasecmp (s1, s2, nLength)==0)  return  (LPSTR) s1;
       s1++;
    }
} /* Tcp4uStrIStr  */
